==> graph_widget-0.0.0.tar.gz: graph_widget-0.0.0/src/graph_widget/static/widget.js <==
import ForceGraph from "https://esm.sh/force-graph";
import { select } from "https://esm.sh/d3-selection";
import { extent } from "https://esm.sh/d3-array";
import { brush } from "https://esm.sh/d3-brush@3";
import { forceManyBody } from "https://esm.sh/d3";
import { scaleLinear, scaleOrdinal, scaleIdentity } from "https://esm.sh/d3-scale";
import { schemeCategory10 } from "https://esm.sh/d3-scale-chromatic";
import RBush from 'https://cdn.jsdelivr.net/npm/rbush/+esm';

let width = "800";
let height = "500";

class MyRBush extends RBush {
    toBBox(node) { return { id: node.id, minX: node.x, minY: node.y, maxX: node.x, maxY: node.y }; }
    compareMinX(a, b) { return a.x - b.x; }
    compareMinY(a, b) { return a.y - b.y; }
}

let local_selected_ids = []
let repulsion = 1;
let node_scale = 1;
let plot;
let tree;
let node_size = 5;
let colour_feature = undefined;
let colour_scale;

function debounce(func, wait) {
    let timeout;
    return function (...args) {
        clearTimeout(timeout);
        timeout = setTimeout(() => func.apply(this, args), wait);
    };
}

const create_rtree = (data) => {
    tree = new MyRBush()
    tree.load(data)
}

function create_plot(el, width, height, data) {
    return ForceGraph()(el)
        .width(width)
        .height(height)
        .graphData(data)
        // .d3Force('charge',forceManyBody().strength(-repulsion))
        .cooldownTime(2000)
        .minZoom(0.001)
        .onEngineStop(() => {
            plot.zoomToFit(400);
            create_rtree(data["nodes"]);
        });
}

const get_feature_type = (value) => {
    if (typeof value === 'number') return 'numeric'; // Check for numeric
    if (typeof value === 'string' && value.startsWith('#')) return 'hash_string'; // Check for strings starting with #
    return 'categorical'; // Default to categorical if not numeric or hash_string
}

// function get_colour_scale(data, colour_feature) {
//     const feature_values = nodes.map(node => node[colour_feature])
//     feature_type = get_feature_type(feature_values[0])
//     if (feature_type == 'numeric') {
//         const domain = extent(feature_values)
//         colour_scale = scaleLinear()
//             .domain(domain)
//             .range(["yellow", "blue"])
//     } else if (feature_type == 'categorical') {
//         const domain = extent(feature_values)
//         colour_scale = scaleOrdinal()
//             .domain(domain)
//             .range(schemeCategory10);
//     } else if (feature_type == 'hash_string') {
//         colour_scale = scaleIdentity()
//     }
// }


function render({ model, el }) {
    const debouncedSaveChanges = debounce(() => model.save_changes(), 300);

    const data = model.get("data");
    repulsion = model.get("repulsion");
    let global_selected_ids = model.get("selected_ids")

    plot = create_plot(el, width, height, data);

    model.on("change:repulsion", () => {
        repulsion = model.get("repulsion");
        plot.d3Force('charge', forceManyBody().strength(-repulsion));
        plot.d3ReheatSimulation();
    });
    model.on("change:node_scale", () => {
        node_size = model.get("node_scale");
        plot.nodeRelSize(node_size * node_scale)
    });
    model.on("change:colour_feature", () => {
        colour_feature = model.get("colour_feature");
        let feature_values = data.nodes.map(node => node[colour_feature])
        let feature_type = get_feature_type(feature_values[0])
        // let colour_scale = undefined;
        colour_scale = ( feature_type == 'numeric')
            ?  scaleLinear().domain(extent(feature_values)).range(["yellow", "blue"])
        : ( feature_type == 'categorical')
            ? colour_scale = scaleOrdinal().domain(extent(feature_values)).range(schemeCategory10)
        : ( feature_type == 'hash_string')
            ? colour_scale = scaleIdentity()
        : undefined


        // if ( feature_type == 'numeric') {
        //     colour_scale = scaleLinear().domain(extent(feature_values)).range(["yellow", "blue"])
        // } else if ( feature_type == 'categorical') {
        //     colour_scale = scaleOrdinal().domain(extent(feature_values)).range(schemeCategory10);
        // } else if ( feature_type == 'hash_string') {
        //     colour_scale = scaleIdentity()
        // }

        plot.nodeColor((d) => {
            return global_selected_ids.includes(d.id)
                ? "yellow"
            : local_selected_ids.includes(d.id)
                ? "rgba(255,0,0,0.5)" // Red for local_selected_ids nodes
            : feature_type === undefined
                ? "steelblue" // Default colour
            : feature_type === 'hash_string'
                ? d[colour_feature] // Use the hash string directly
            : colour_scale(d[colour_feature])})
    });
    model.on("change:selected_ids", () => {
        global_selected_ids = model.get("selected_ids");
        plot.nodeColor((d) =>
            global_selected_ids.includes(d.id)
                ? "red"
            : local_selected_ids.includes(d.id)
                ? "rgba(255,0,0,0.5)"
                : d["colour_hex"],
        );
    });

    // model.on("change:selected_nodes", () => {
    //     local_selected_ids = model.get("selected_nodes");
    //     plot.nodeColor((d) =>
    //         global_selected_ids.includes(d.id)
    //             ? "red"
    //         : selected.includes(d.id)
    //             ? "rgba(255,0,0,0.5)"
    //             : d["colour_hex"],
    //     );
    // });

    let brush_active = false;

    let container = el.querySelector(".force-graph-container")

    // Create an overlay for brushing
    let overlay = select(container)
        .append("svg")
        .attr("id", "overlay")
        .style("position", "absolute")
        .style("top", 0)
        .style("left", 0)
        .style("width", "100%")
        .style("height", "100%")
        .style("pointer-events", "none")

    let activate_brush = () => {
        brush_active = true;
        el.style.pointerEvents = "none"
        overlay.style.pointerEvents = "auto"

        if (!overlay.select("#brush_group").empty()) return;
        overlay.insert("g", ":first-child")
            .attr("id", "brush_group")
            .attr("class", "brush")
            .call(my_brush)
    }
    let disactivate_brush = () => {
        brush_active = false;
        el.style.pointerEvents = "auto"
        overlay.style.pointerEvents = "none"

        overlay.select("#brush_group").remove();
    }

    let brushed = (event) => {
        if (event.selection) {
            brush_active = true;
            let [[x0_screen, y0_screen], [x1_screen, y1_screen]] =
                event.selection;
            let corner0 = plot.screen2GraphCoords(x0_screen, y0_screen);
            let corner1 = plot.screen2GraphCoords(x1_screen, y1_screen);
            let bbox = {
                minX: Math.min(corner0.x, corner1.x),
                minY: Math.min(corner0.y, corner1.y),
                maxX: Math.max(corner0.x, corner1.x),
                maxY: Math.max(corner0.y, corner1.y)
            };
            let selectedNodes = tree.search(bbox)
            local_selected_ids = selectedNodes.map(node => node.id);
            model.set("selected_ids", local_selected_ids);
            console.log(local_selected_ids)
            debouncedSaveChanges();
        }
    };

    let my_brush = brush()
        .filter((event) => {
            // Prevent contextual menu on Ctrl-click (macOS)
            if (event.ctrlKey && event.button === 0) {
                event.preventDefault();
            }
            return (
                !event.button && // Ignore mouse buttons other than left-click
                (event.metaKey || event.ctrlKey || event.target.__data__.type !== "overlay")
            )
        })
        .extent([[0, 0], [width, height]])
        .on("start brush end", brushed);

    window.addEventListener("keydown", (e) => {
        if (e.metaKey && !brush_active) { activate_brush() }
    })
    window.addEventListener("dblclick", (e) => {
        if (brush_active) { disactivate_brush() }
    })
}

export default { render };

==> graph_widget-0.1.3.tar.gz: graph_widget-0.1.3/src/graph_widget/static/widget.js <==
import ForceGraph from "https://esm.sh/force-graph";
import { select } from "https://esm.sh/d3-selection";
import { extent, min, max } from "https://esm.sh/d3-array";
import { brush } from "https://esm.sh/d3-brush@3";
import { forceManyBody } from "https://esm.sh/d3";
import { scaleLinear, scaleOrdinal, scaleIdentity } from "https://esm.sh/d3-scale";
import { schemeCategory10 } from "https://esm.sh/d3-scale-chromatic";
import RBush from 'https://cdn.jsdelivr.net/npm/rbush/+esm';

let width = "800";
let height = "500";

class MyRBush extends RBush {
    toBBox(node) { return { id: node.id, minX: node.x, minY: node.y, maxX: node.x, maxY: node.y }; }
    compareMinX(a, b) { return a.x - b.x; }
    compareMinY(a, b) { return a.y - b.y; }
}

let local_selected_ids = []
let repulsion = 1;
let node_scale = 1;
let colour_scale_type = "";
let plot;
let tree;
let node_size = 5;
let colour_feature = undefined;
let colour_scale;

function debounce(func, wait) {
    let timeout;
    return function (...args) {
        clearTimeout(timeout);
        timeout = setTimeout(() => func.apply(this, args), wait);
    };
}

const create_rtree = (data) => {
    tree = new MyRBush()
    tree.load(data)
}

function create_plot(el, width, height, data) {
    return ForceGraph()(el)
        .width(width)
        .height(height)
        .graphData(data)
        // .d3Force('charge',forceManyBody().strength(-repulsion))
        .cooldownTime(5000)
        .warmupTicks(10)
        .d3AlphaDecay(0.001)
        .minZoom(0.001)
        .onEngineStop(() => {
            plot.zoomToFit(400);
            create_rtree(data["nodes"]);
        });
}

const get_feature_type = (value) => {
    if (typeof value === 'number') return 'numeric'; // Check for numeric
    if (typeof value === 'string' && value.startsWith('#')) return 'hash_string'; // Check for strings starting with #
    return 'categorical'; // Default to categorical if not numeric or hash_string
}

function render({ model, el }) {
    const debouncedSaveChanges = debounce(() => model.save_changes(), 300);

    const data = model.get("data");
    repulsion = model.get("repulsion");
    node_size = model.get("node_scale");
    colour_scale_type = model.get("colour_scale_type");
    colour_feature = model.get("colour_feature");
    let global_selected_ids = model.get("selected_ids")

    plot = create_plot(el, width, height, data);
    model.on("change:repulsion", () => {
        repulsion = model.get("repulsion");
        plot.d3Force('charge', forceManyBody().strength(-repulsion));
        plot.d3ReheatSimulation();
    });
    model.on("change:node_scale", () => {
        node_size = model.get("node_scale");
        plot.nodeRelSize(node_size * node_scale)
        plot.d3ReheatSimulation();
    });
    model.on("change:colour_feature", () => {
        colour_feature = model.get("colour_feature");
        let feature_values = data.nodes.map(node => node[colour_feature])
        let feature_type = get_feature_type(feature_values[0])
        if ( feature_type == "numeric" ) {
            if ( colour_scale_type == "diverging" ) {
                colour_scale = scaleLinear().domain([min(feature_values), 0, max(feature_values)]).range(["blue","#eeeeee","red"])
            } else {
                colour_scale = scaleLinear().domain(extent(feature_values)).range(["yellow", "blue"])
            }
        } else if ( feature_type == "categorical" ) {
            colour_scale = scaleOrdinal().domain(extent(feature_values)).range(schemeCategory10)
        } else if ( feature_type == "hash_string" ) {
            colour_scale = scaleIdentity()
        } else {
            colour_scale = undefined
        }

        plot.nodeColor((d) => {
            return global_selected_ids.includes(d.id)
                ? "yellow"
            : local_selected_ids.includes(d.id)
                ? "rgba(255,0,0,0.5)" // Red for local_selected_ids nodes
            : feature_type === undefined
                ? "steelblue" // Default colour
            : feature_type === 'hash_string'
                ? d[colour_feature] // Use the hash string directly
            : colour_scale(d[colour_feature])})
    });
    model.on("change:colour_scale_type", () => {
        console.log("COLOUR SCALE TYPE CHANGED")
        colour_feature = model.get("colour_feature");
        colour_scale_type = model.get("colour_scale_type")
        let feature_values = data.nodes.map(node => node[colour_feature])
        let feature_type = get_feature_type(feature_values[0])
        if ( feature_type == "numeric" ) {
            if ( colour_scale_type == "diverging" ) {
                colour_scale = scaleLinear().domain([min(feature_values), 0, max(feature_values)]).range(["blue","#eeeeee","red"])
            } else {
                colour_scale = scaleLinear().domain(extent(feature_values)).range(["yellow", "blue"])
            }
        }
        plot.nodeColor((d) => {
            return global_selected_ids.includes(d.id)
                ? "yellow"
            : local_selected_ids.includes(d.id)
                ? "rgba(255,0,0,0.5)" // Red for local_selected_ids nodes
            : feature_type === undefined
                ? "steelblue" // Default colour
            : feature_type === 'hash_string'
                ? d[colour_feature] // Use the hash string directly
            : colour_scale(d[colour_feature])})
    })
    model.on("change:selected_ids", () => {
        global_selected_ids = model.get("selected_ids");
        plot.nodeColor((d) =>
            global_selected_ids.includes(d.id)
                ? "red"
            : local_selected_ids.includes(d.id)
                ? "rgba(255,0,0,0.5)"
                : d["colour_hex"],
        );
    });

    let brush_active = false;

    let container = el.querySelector(".force-graph-container")

    // Create an overlay for brushing
    let overlay = select(container)
        .append("svg")
        .attr("id", "overlay")
        .style("position", "absolute")
        .style("top", 0)
        .style("left", 0)
        .style("width", "100%")
        .style("height", "100%")
        .style("pointer-events", "none")

    let activate_brush = () => {
        brush_active = true;
        el.style.pointerEvents = "none"
        overlay.style.pointerEvents = "auto"

        if (!overlay.select("#brush_group").empty()) return;
        overlay.insert("g", ":first-child")
            .attr("id", "brush_group")
            .attr("class", "brush")
            .call(my_brush)
    }
    let disactivate_brush = () => {
        brush_active = false;
        el.style.pointerEvents = "auto"
        overlay.style.pointerEvents = "none"

        overlay.select("#brush_group").remove();
    }

    let brushed = (event) => {
        if (event.selection) {
            brush_active = true;
            let [[x0_screen, y0_screen], [x1_screen, y1_screen]] =
                event.selection;
            let corner0 = plot.screen2GraphCoords(x0_screen, y0_screen);
            let corner1 = plot.screen2GraphCoords(x1_screen, y1_screen);
            let bbox = {
                minX: Math.min(corner0.x, corner1.x),
                minY: Math.min(corner0.y, corner1.y),
                maxX: Math.max(corner0.x, corner1.x),
                maxY: Math.max(corner0.y, corner1.y)
            };
            let selectedNodes = tree.search(bbox)
            local_selected_ids = selectedNodes.map(node => node.id);
            model.set("selected_ids", local_selected_ids);
            // console.log(local_selected_ids)
            debouncedSaveChanges();
        }
    };

    let my_brush = brush()
        .filter((event) => {
            // Prevent contextual menu on Ctrl-click (macOS)
            if (event.ctrlKey && event.button === 0) {
                event.preventDefault();
            }
            return (
                !event.button && // Ignore mouse buttons other than left-click
                (event.metaKey || event.ctrlKey || event.target.__data__.type !== "overlay")
            )
        })
        .extent([[0, 0], [width, height]])
        .on("start brush end", brushed);

    window.addEventListener("keydown", (e) => {
        if (e.metaKey && !brush_active) { activate_brush() }
    })
    window.addEventListener("dblclick", (e) => {
        if (brush_active) { disactivate_brush() }
    })
}

export default { render };
